@@ -43,10 +43,10 @@ G_BEGIN_DECLS
 void g_on_error_query (const gchar *prg_name);
 void g_on_error_stack_trace (const gchar *prg_name);
 
-/* Hacker macro to place breakpoints for elected machines.
+/* Hacker macro to place breakpoints for selected machines.
  * Actual use is strongly discouraged of course ;)
  */
-#if defined (__i386__) && defined (__GNUC__) && __GNUC__ >= 2
+#if (defined (__i386__) || defined (__x86_64__)) && defined (__GNUC__) && __GNUC__ >= 2
 #  define G_BREAKPOINT()	G_STMT_START{ __asm__ __volatile__ ("int $03"); }G_STMT_END
 #elif defined (_MSC_VER) && defined (_M_IX86)
 #  define G_BREAKPOINT()	G_STMT_START{ __asm int 3h }G_STMT_END
