@@ -43,11 +43,18 @@
 #include <stdio.h>
 #include <string.h>
 #include <sys/types.h>
+#ifdef HAVE_UNISTD_H
 #include <unistd.h>
+#endif
 
+#include "galias.h"
 #include "glib.h"
 #include "gthreadinit.h"
 
+#ifdef G_OS_WIN32
+#include <process.h>		/* For getpid() */
+#endif
+
 G_LOCK_DEFINE_STATIC (global_random);
 static GRand* global_random = NULL;
 
@@ -131,6 +138,8 @@ g_rand_new_with_seed (guint32 seed)
  * Creates a new random number generator initialized with @seed.
  * 
  * Return value: the new #GRand.
+ *
+ * Since: 2.4
  **/
 GRand*
 g_rand_new_with_seed_array (const guint32 *seed, guint seed_length)
@@ -202,7 +211,11 @@ g_rand_new (void)
       seed[0] = now.tv_sec;
       seed[1] = now.tv_usec;
       seed[2] = getpid ();
+#ifdef G_OS_UNIX
       seed[3] = getppid ();
+#else
+      seed[3] = 0;
+#endif
     }
 
   return g_rand_new_with_seed_array (seed, 4);
@@ -231,6 +244,8 @@ g_rand_free (GRand* rand)
  * replaying later.
  *
  * Return value: the new #GRand.
+ *
+ * Since: 2.4
  **/
 GRand *
 g_rand_copy (GRand* rand)
@@ -299,6 +314,8 @@ g_rand_set_seed (GRand* rand, guint32 seed)
  * first 624 values are taken.  This function is useful
  * if you have many low entropy seeds, or if you require more then
  * 32bits of actual entropy for your application.
+ *
+ * Since: 2.4
  **/
 void
 g_rand_set_seed_array (GRand* rand, const guint32 *seed, guint seed_length)
