@@ -19,6 +19,9 @@
  */
 
 #include "config.h"
+#include "galias.h"
+
+#include "glib.h"
 
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -29,12 +32,12 @@
 #endif
 
 #ifdef G_OS_WIN32
+#include <errno.h>
 #include <wchar.h>
+#include <direct.h>
 #include <io.h>
 #endif
 
-#include "galias.h"
-#include "glib.h"
 #include "gstdio.h"
 
 #if !defined (G_OS_UNIX) && !defined (G_OS_WIN32)
@@ -48,12 +51,17 @@
  * @flags: as in open()
  * @mode: as in open()
  *
- * A wrapper for the POSIX open() function. The open() function is used 
- * to convert a pathname into a file descriptor.
+ * A wrapper for the POSIX open() function. The open() function is
+ * used to convert a pathname into a file descriptor. Note that on
+ * POSIX systems file descriptors are implemented by the operating
+ * system. On Windows, it's the C library that implements open() and
+ * file descriptors. The actual Windows API for opening files is
+ * something different.
  *
  * See the C library manual for more details about open().
  *
- * Returns: a new file descriptor, or -1 if an error occurred
+ * Returns: a new file descriptor, or -1 if an error occurred. The
+ * return value can be used exactly like the return value from open().
  * 
  * Since: 2.6
  */
@@ -198,7 +206,7 @@ g_mkdir (const gchar *filename,
  * 
  * See the C library manual for more details about stat().
  *
- * Returns: 0 if the directory was successfully created, -1 if an error 
+ * Returns: 0 if the information was successfully retrieved, -1 if an error 
  *    occurred
  * 
  * Since: 2.6
@@ -235,6 +243,37 @@ g_stat (const gchar *filename,
 #endif
 }
 
+/**
+ * g_lstat: 
+ * @filename: a pathname in the GLib file name encoding
+ * @buf: a pointer to a <structname>stat</structname> struct, which
+ *    will be filled with the file information
+ *
+ * A wrapper for the POSIX lstat() function. The lstat() function is
+ * like stat() except that in the case of symbolic links, it returns
+ * information about the symbolic link itself and not the file that it
+ * refers to. If the system does not support symbolic links g_lstat()
+ * is identical to g_stat().
+ * 
+ * See the C library manual for more details about lstat().
+ *
+ * Returns: 0 if the information was successfully retrieved, -1 if an error 
+ *    occurred
+ * 
+ * Since: 2.6
+ */
+int
+g_lstat (const gchar *filename,
+	 struct stat *buf)
+{
+#ifdef HAVE_LSTAT
+  /* This can't be Win32, so don't do the widechar dance. */
+  return lstat (filename, buf);
+#else
+  return g_stat (filename, buf);
+#endif
+}
+
 /**
  * g_unlink:
  * @filename: a pathname in the GLib file name encoding
